var TimeChart = null;

(function($) {
	
TimeChart = Class.extend({
	
	/**
	 * Constructor
	 */
    init: function (options) {
        this.defaults = {
        	chartData: new Array(),
        	leftLineText: "",
        	rightLineText: "Zeit",
        	bottomLineText: "Runde"
        };
        
        this.options = $.extend(this.defaults, options);
        
        this.chartSet = new Array();
        this.chartPaper = {};
        this.containerWidth = 0;
    	this.containerHeight = 0;
    	this.scaleElements = {};
    	this.scalePaddingTop = 10;
    	this.roundTimeMax = 0;
    	this.roundTimeDefault = 0;
        this.titleFontSize = 24;
    	this.textFontSize = 16;
    	this.lineStrength = 3;
    	this.dotStrength = 6;
    	this.RGBblue = 'rgb(18,191,231)';
    	this.RGBgrey = 'rgb(87,87,87)';
    	this.RGBlightGrey = 'rgb(238,238,238)';
    	this.RGBgreen = 'rgb(149,193,30)';
    	this.RGByellow = 'rgb(255,197,0)';
    	this.RGBorange = 'rgb(255,129,0)';
    	this.currentGroup = 0;
    	
        this.initialize();
    },
    
    initialize: function() {
    	var self = this;
    	/*
    	window.onresize = function() {
    		self.resize();
    	};
    	*/
    	var windowHeight = window.innerHeight;
    	var chartHeight = 0.4 * windowHeight;
    	document.getElementById('chart').style.height = chartHeight + "px";
    	
    	this.roundTimeMax = this.findMaxTime(this.defaults.chartData);
    	this.roundTimeDefault = convertToSeconds({minutes: this.options.roundMinutes, seconds: this.options.roundSeconds});
		
		// Get the current width & height of chart container div, for rendering the graphics according to viewport size
		this.containerWidth = document.getElementById('chart').offsetWidth;
		this.containerHeight = document.getElementById('chart').offsetHeight;
		
		if(this.containerWidth < 900 && this.containerWidth > 600) {
			this.titleFontSize= 16;
			this.textFontSize = 12;
			this.lineStrength = 2;
			this.dotStrength = 4;
		}
		else if (this.containerWidth < 600) {
			this.titleFontSize = 13;
			this.textFontSize = 10;
			this.lineStrength = 1;
			this.dotStrength = 3;
		}
		else {
			this.titleFontSize = 24;
			this.textFontSize = 16;
			this.lineStrength = 3;
			this.dotStrength = 6;
		}
		var gr = ( parseInt(this.currentGroup) + parseInt(1) );
		$('.groupID').text(gr);
		this.chartPaper = Raphael('chart', '100%', '100%');
		this.chartSet = this.chartPaper.set();
		this.drawChartScale();
		this.drawChartData(this.defaults.chartData[this.currentGroup]);
		this.addGroupButtons();
	},
	
	drawChartScale: function() {
		// Set the dimensions of chart scale
		var scaleHeight = this.containerHeight*0.95;
		var scaleWidth = this.containerWidth*0.94;
		var leftLinePosLeft = this.containerWidth*0.05;
		var leftLine = "M"+leftLinePosLeft+" "+this.scalePaddingTop+"V"+scaleHeight;
		var bottomLinePosLeft = leftLinePosLeft * 0.85;
		var bottomLinePosTop = scaleHeight - leftLinePosLeft * 0.15;
		var bottomLine = "M"+bottomLinePosLeft+" "+bottomLinePosTop+"H"+scaleWidth;
		var rightLinePosLeft = scaleWidth - leftLinePosLeft * 0.15;
		var rightLine = "M"+rightLinePosLeft+" "+this.scalePaddingTop+"V"+scaleHeight;
		var lines = new Array(leftLine, bottomLine, rightLine);
		
		this.scaleElements = {
				scaleWidth: scaleWidth,
				scaleHeight: scaleHeight,
				scalePaddingTop: this.scalePaddingTop,
				leftLinePosLeft: leftLinePosLeft,
				bottomLinePosLeft: bottomLinePosLeft,
				bottomLinePosTop: bottomLinePosTop,
				rightLinePosLeft: rightLinePosLeft
		};
		
		// Set scale step lines
		var steps = this.roundTimeMax * 4;
		var stepSeconds = 15;
		var stepText = "";
		var stepsLeft = steps;
		var step = (bottomLinePosTop - this.scalePaddingTop) / steps;
		var top = this.scalePaddingTop + 1;
		var right = scaleWidth - leftLinePosLeft * 0.15;
		var left = right - 15;
		var scaleStepLeft = "";
		var scaleStepRight = "";
		for(var i=0; i < steps; i++) {
			stepsLeft = steps - i;
			scaleStepLeft = "M"+leftLinePosLeft+" "+top+"H"+(leftLinePosLeft+15);
			scaleStepRight = "M"+left+" "+top+"H"+right;
			lines.push(scaleStepLeft,scaleStepRight);
			
			// Step text
			if(i * stepSeconds == 0 || i * stepSeconds == 60)
				stepText = Math.floor( stepsLeft / 4 ) + " Min";
			else
				stepText = (stepsLeft % 4) * (60 / 4) + " Sek";
			
			this.chartSet.push(this.chartPaper.text(0, top - (this.textFontSize / 5), stepText)
					.attr( 'fill', this.RGBgrey )
					.attr( 'font-size', this.textFontSize )
					.attr( 'text-anchor', 'start' ));
			
			top = top + step - 0.5;
		}
		
		// Draw the scale lines & chart text
		this.chartSet.push(this.chartPaper.path(lines).attr( 'stroke-width', this.lineStrength ).attr( 'stroke', this.RGBgrey ));
		
		this.chartSet.push(this.chartPaper.text(right*1.005, this.scalePaddingTop, this.roundTimeMax+" Min")
				.attr( 'fill', this.RGBgrey )
				.attr( 'font-size', this.textFontSize )
				.attr( 'text-anchor', 'start' ));
		
		this.chartSet.push(this.chartPaper.text(scaleWidth*1.005, scaleHeight*0.54, this.defaults.rightLineText)
				.attr( 'transform', 'T0R-90T0' )
				.attr( 'fill', this.RGBgrey )
				.attr( 'font-size', this.titleFontSize )
				.attr( 'text-anchor', 'start' ));
		/*
		this.chartSet.push(this.chartPaper.text(leftLinePosLeft*1.8, bottomLinePosTop + ( (this.containerHeight-bottomLinePosTop)*0.50 ), 
				this.defaults.bottomLineText)
				.attr( 'fill', this.RGBgrey )
				.attr( 'font-size', this.textFontSize )
				.attr( 'text-anchor', 'start' ));
		*/
	},
	
	drawChartData: function(groupTimes) {
		var self = this;
		var scaleValues = this.scaleElements;
		var pixelsPerSecond = (scaleValues.bottomLinePosTop - 10) / (this.roundTimeMax * 60);
		var roundSeconds;
		var scaleRealWidth = scaleValues.rightLinePosLeft - scaleValues.leftLinePosLeft;
		var timeBarWidth = scaleRealWidth / this.options.amountRounds * 0.6;
		var timeBarHeight, barPositionTop, barPositionLeft;
		var barGapWidth = ( scaleRealWidth - ( this.options.amountRounds * timeBarWidth ) ) / ( parseInt(this.options.amountRounds) + 1 );
		var timeColor = this.RGBgreen;
		var overTimeColor = this.RGBorange;
		var timeLine, timeLinePosTop, bar;
		
		for (var i = 0; i < groupTimes.length; i++) {
			// Calculate the height and position of time bars
			timeBarHeight = 0;
			roundSeconds = convertToSeconds(groupTimes[i]);
			barPositionLeft = scaleValues.leftLinePosLeft + barGapWidth + (i * timeBarWidth) + ( i * barGapWidth );
			barPositionTop = scaleValues.bottomLinePosTop - (roundSeconds * pixelsPerSecond);
			barPositionTop2 = scaleValues.bottomLinePosTop - (roundSeconds * pixelsPerSecond);
			
			// Draw a separate bar for overtime, if time was longer than the default time for one round
			if(roundSeconds > self.roundTimeDefault) {
				timeBarHeight = (roundSeconds - self.roundTimeDefault) * pixelsPerSecond;
				
				bar = this.chartPaper.rect(barPositionLeft, barPositionTop, timeBarWidth, timeBarHeight);
				self.chartSet.push(bar.attr( 'stroke', overTimeColor ).attr( 'fill', overTimeColor ));
				barPositionTop2 = barPositionTop + timeBarHeight;
			}
			timeBarHeight = (roundSeconds * pixelsPerSecond) - timeBarHeight - 2;
			// Draw time bar (the part less than the default round time)
			bar = this.chartPaper.rect(barPositionLeft, barPositionTop2, timeBarWidth, timeBarHeight);
			self.chartSet.push(bar.attr( 'stroke', timeColor ).attr( 'fill', timeColor ));
			
			// Round time above each round time bar
			this.chartSet.push(this.chartPaper.text(barPositionLeft, barPositionTop - this.textFontSize * 0.55, groupTimes[i].minutes+" Min "+groupTimes[i].seconds+" Sek")
					.attr( 'fill', this.RGBgrey )
					.attr( 'font-size', this.textFontSize )
					.attr( 'text-anchor', 'start' ));
			
			// Round number text under each round time bar
			this.chartSet.push(this.chartPaper.text(barPositionLeft, scaleValues.bottomLinePosTop + 15, "Runde "+(i+1))
					.attr( 'fill', this.RGBgrey )
					.attr( 'font-size', this.textFontSize )
					.attr( 'text-anchor', 'start' ));
		}
		
		// Draw default time line
		timeLinePosTop = scaleValues.bottomLinePosTop - (this.roundTimeDefault * pixelsPerSecond);
		timeLineWidth = scaleValues.scaleWidth - (scaleValues.leftLinePosLeft * 0.15);
		timeLine = "M"+scaleValues.leftLinePosLeft+" "+timeLinePosTop+"H"+timeLineWidth;
		this.chartSet.push(this.chartPaper.path(timeLine).attr( 'stroke-width', this.lineStrength + 1 ).attr( 'stroke', this.RGBblue ));
	},
	
	addGroupButtons: function () {
		var self = this;
		var groupsButtonContainer = document.getElementById('chartGroups');
		var att;
		var gc = 0;
		
		for(var i = 0; i < this.defaults.chartData.length; i++) {
			groupButton = document.createElement("button");
			groupButton.id = "chart_"+i;
			groupButton.className = "btChartGroup bt";
			groupButton.textContent = "Gruppe "+(i+1);
			groupsButtonContainer.appendChild(groupButton);
			att = document.createAttribute("gc");
			att.value = i;
			groupButton.setAttributeNode(att);
			if(i > 0)
				groupButton.style.marginLeft = "20px";
			
			if(i == this.currentGroup)
				groupButton.style.fontWeight = "bold";
		}
		$('.btChartGroup').click(function(){
			gc = $(this).attr('gc');
			self.currentGroup = gc;
			document.getElementById('chart').innerHTML = "";
			document.getElementById('chartGroups').innerHTML = "";
			self.initialize();
		});
	},
	
	findMaxTime: function (arr) {
		var secondsTotal = 0;
		var max = 0;
		if(arr.length > 0) {
			for(var i = 0; i < arr.length; i++) {
				for(var x = 0; x < arr[i].length; x++) {
					secondsTotal = arr[i][x].minutes*60 + arr[i][x].seconds;
					if(secondsTotal > max)
						max = secondsTotal;
				}
			}
			max = Math.floor(max / 60) + 1;
		}
		return max;
	}
	/*
	resize: function() {
		document.getElementById('chart').innerHTML = "";
		document.getElementById('chartGroups').innerHTML = "";
		this.initialize();
	}
	*/
	
});

function convertToSeconds (obj) {
	return obj.minutes * 60 + obj.seconds;
}

})(jQuery);